﻿// PGN Base
//
//
//+++++++1+++++++++2+++++++++3+++++++++4+++++++++5+++++++++6+++++++++7+++++++++8+++++++9+++++++++0+++++++++1+++++++++2++

using System;
using UnityEngine;
using System.Collections;


public class PGX_Grid : MonoBehaviour
{

	// get the height
	public static float PGX_GetHeight(ref Vector3 _pos)
	{
		float x = _pos.x - 1000;
		float z = _pos.z - 1000;
		
		float y = 200.0F * (float)Math.Exp( -0.00001 * (x*x + z*z));
		
		return y+ 12.0F;
	}

	//get the uv-u value
	public static float PGX_GetUV(float base_y, float test_y)
	{
		float t = (test_y - base_y) * 0.01F + 0.5F;

		if(0.001F >t)
			t = 0.001F;
		else if(.99F < t)
			t = .99F;

		return t;
	}

	// get the player height
	public static float PGX_GetPlayerH()
	{
		return 48.0F;
	}


	protected Material 			m_mtl;

	protected MeshFilter		m_msf;
	protected MeshRenderer		m_rnd;
	protected Mesh				m_msh;

	protected float				m_fT;					// thickness
	protected float				m_fW;					// width
	protected int				m_nX;					// count x direction
	protected int				m_nZ;					// count z direction

	protected Vector3			m_vcP;					// start position
	protected Vector3			m_vcX;					// side direction
	protected Vector3			m_vcZ;					// forward direction


	// Use this for initialization
	void Start()
	{
		////////////////////////////////////////////////////////////////////////
		// setup the mesh filter and mesh renderer

		m_msf = gameObject.GetComponent<MeshFilter>();
		if(null == m_msf)
			m_msf = gameObject.AddComponent<MeshFilter>();

		m_rnd = gameObject.GetComponent<MeshRenderer>();
		if(null == m_rnd)
			m_rnd = gameObject.AddComponent<MeshRenderer>();


		Texture tx0   = Resources.LoadAssetAtPath("Assets/pgx_grid/pgx_grid_c.png", typeof(Texture)) as Texture;
		Texture tx1   = Resources.LoadAssetAtPath("Assets/pgx_grid/pgx_grid_w.png", typeof(Texture)) as Texture;
		Shader shader = Resources.LoadAssetAtPath("Assets/pgx_grid/PGX_Grid.shader", typeof(Shader)) as Shader;

		if(null == tx0)
			Debug.LogError("Cann't load tx0");

		if(null == tx1)
			Debug.LogError("Cann't load tx1");

		if(null == shader)
			Debug.LogError("Cann't load shader");


		//Shader shader = Shader.Find("Custom/PGX_Grid");



		m_mtl = new Material( shader);
		m_mtl.SetTexture("_tex0", tx0);
		m_mtl.SetTexture("_tex1", tx1);

		m_rnd.sharedMaterial = m_mtl;

		////////////////////////////////////////////////////////////////////////
		// create mesh

		m_msh = new Mesh();
        m_msh.name = "grid";
		m_msf.sharedMesh = m_msh;



		// test...
		//Vector3[] vtP = new Vector3[4];
		//Vector2[] vtT = new Vector2[4];
		//int    [] tri = new int[3 * 2];

		//vtP[0] = new Vector3(-10.0F, -10.0F, 0.0F);
		//vtP[1] = new Vector3(-10.0F,  10.0F, 0.0F);
		//vtP[2] = new Vector3( 10.0F, -10.0F, 0.0F);
		//vtP[3] = new Vector3( 10.0F,  10.0F, 0.0F);

		//vtT[0] = new Vector2( 0.0F,  1.0F);
		//vtT[1] = new Vector2( 0.0F,  0.0F);
		//vtT[2] = new Vector2( 1.0F,  1.0F);
		//vtT[3] = new Vector2( 1.0F,  0.0F);

		//tri[0] = 0; tri[1] = 1; tri[2] = 2;
		//tri[3] = 3; tri[4] = 2; tri[5] = 1;

		//m_msh.vertices = vtP;
		//m_msh.uv       = vtT;
		//m_msh.triangles= tri;

		// test..
		Vector3 pos = new Vector3(-100,20,0);
		Vector3 axX = new Vector3( 1,0,0);
		Vector3 axZ = new Vector3( 0,0,1);

		this.CreateMesh(4.0F, 50.0F, 50, 50, ref pos, ref axX, ref axZ);
	}

	
	void Update()
	{
	}


	// create grid mesh
	public void CreateMesh(
				float thick, float with							// grid thickness, grid width
				, int   nX, int nZ								// count for side(x) and forward(z) direction
				, ref Vector3 pos								// start position
				, ref Vector3 vcX								// x direction
				, ref Vector3 vcZ								// z direciton
				)
	{
		m_fT	= thick;
		m_fW	= with;
		m_nX	= nX;
		m_nZ	= nZ;

		int nTri = (m_nX * (m_nZ+1) + (m_nX+1) * m_nZ) * 4 * 3;	// triangles
		int nVtx = (m_nX+1) * (m_nZ+1)* 4 * 2;					// vertices

		Vector3[] vtP = new Vector3[nVtx];						// create the position buffer
		Vector2[] vtT = new Vector2[nVtx];						// create the uv buffer
		int    [] tri = new int[nTri];							// create the index buffer
	
		//Debug.Log("Triangles: " + nTri +" vertices: " + nVtx);


		// binding vertex and index buffer
		m_msh.vertices  = vtP;
		m_msh.uv        = vtT;
		m_msh.triangles = tri;

		this.Setup(ref pos, ref vcX, ref vcZ);
	}


	// setup the grid start position, x direction, z direciton
	public void Setup(ref Vector3 pos, ref Vector3 vcX, ref Vector3 vcZ)
	{
		int i=0,k=0, n=0;
		int	bgn  = 0;

		// get the position buffer of vertex
		Vector3[] vtP = m_msh.vertices;
		Vector2[] vtT = m_msh.uv;
		int    [] tri = m_msh.triangles;

		m_vcP	= pos;											// start position
		m_vcX	= vcX;											// side direction
		m_vcZ	= vcZ;											// forward direction

		m_vcX.y =0;												// make up the x,z plane direction
		m_vcZ.y =0;
		m_vcX.Normalize();										// normalize
		m_vcZ.Normalize();


		// setup indices(z-direction)
		n = 0;
		for(k=0; k<=m_nZ; ++k)
		{
			for(i=0; i<m_nX; ++i)
			{
				bgn = ((m_nX+1) * k + i) * 4;

				tri[n++] = bgn + 0;
				tri[n++] = bgn + 2;
				tri[n++] = bgn + 4;

				tri[n++] = bgn + 6;
				tri[n++] = bgn + 4;
				tri[n++] = bgn + 2;

				tri[n++] = bgn + 1;
				tri[n++] = bgn + 3;
				tri[n++] = bgn + 5;

				tri[n++] = bgn + 7;
				tri[n++] = bgn + 5;
				tri[n++] = bgn + 3;
			}
		}

		// setup indices(x-direction)
		for(i=0; i<=m_nX; ++i)
		{
			for(k=0; k<m_nZ; ++k)
			{
				//bgn = ((m_nZ+1) * i + k)*4;
				bgn = ((m_nZ+1) * i + k)*4  + (m_nX+1) * (m_nZ+1)* 4;

				tri[n++] = bgn + 0;
				tri[n++] = bgn + 2;
				tri[n++] = bgn + 4;

				tri[n++] = bgn + 6;
				tri[n++] = bgn + 4;
				tri[n++] = bgn + 2;

				tri[n++] = bgn + 1;
				tri[n++] = bgn + 3;
				tri[n++] = bgn + 5;

				tri[n++] = bgn + 7;
				tri[n++] = bgn + 5;
				tri[n++] = bgn + 3;
			}
		}


		Vector3 vcP;
		// setup the x-direction position
		for(k=0; k<=m_nZ; ++k)
		{
			for(i=0; i<=m_nX; ++i)
			{
				bgn = ((m_nX+1) * k + i)*4;

				vcP = m_vcP + (i * m_fW) * m_vcX + (k * m_fW) * m_vcZ;

				vtP[bgn+0] = vcP;
				vtP[bgn+1] = vcP;
				vtP[bgn+2] = vcP;
				vtP[bgn+3] = vcP;

				vtP[bgn+0].y -= m_fT;
				vtP[bgn+1]   -= m_fT * m_vcZ;
				vtP[bgn+2].y += m_fT;
				vtP[bgn+3]   += m_fT * m_vcZ;

				// setup the uv-v to 1.0 or 0.0
				vtT[bgn+0].y = 1.0F;
				vtT[bgn+1].y = 1.0F;
				vtT[bgn+2].y = 0.0F;
				vtT[bgn+3].y = 0.0F;

				vtT[bgn+0].x = i * 0.125F;
				vtT[bgn+1].x = i * 0.125F;
				vtT[bgn+2].x = i * 0.125F;
				vtT[bgn+3].x = i * 0.125F;
			}
		}

		// setup the z-direction position
		for(i=0; i<=m_nX; ++i)
		{
			for(k=0; k<=m_nZ; ++k)
			{
				bgn = ((m_nZ+1) * i + k)*4  + (m_nX+1) * (m_nZ+1)* 4;

				vcP = m_vcP + (i * m_fW) * m_vcX + (k * m_fW) * m_vcZ;

				vtP[bgn+0] = vcP;
				vtP[bgn+1] = vcP;
				vtP[bgn+2] = vcP;
				vtP[bgn+3] = vcP;

				vtP[bgn+0].y -= m_fT;
				vtP[bgn+1]   += m_fT * m_vcX;
				vtP[bgn+2].y += m_fT;
				vtP[bgn+3]   -= m_fT * m_vcX;

				// setup the uv-v to 1.0 or 0.0
				vtT[bgn+0].y = 1.0F;
				vtT[bgn+1].y = 1.0F;
				vtT[bgn+2].y = 0.0F;
				vtT[bgn+3].y = 0.0F;

				vtT[bgn+0].x = k * 0.125F;
				vtT[bgn+1].x = k * 0.125F;
				vtT[bgn+2].x = k * 0.125F;
				vtT[bgn+3].x = k * 0.125F;
			}
		}


		// binding vertex and index buffer
		m_msh.vertices  = vtP;
		m_msh.uv        = vtT;
		m_msh.triangles = tri;

		SetupHeight();


		// finally, optimizing
		m_msh.Optimize();
		m_msh.RecalculateNormals();
		m_msh.RecalculateBounds();
	}


	protected void SetupHeight()
	{
		Vector3 vcP;
		float hgt = 0.0F;
		float uvt = 0.0F;
		int i=0, k=0;
		int	bgn  = 0;

		float base_y = PGX_GetPlayerH();


		// get the position buffer of vertex
		Vector3[] vtP = m_msh.vertices;
		Vector2[] vtT = m_msh.uv;

		for(k=0; k<=m_nZ; ++k)
		{
			for(i=0; i<=m_nX; ++i)
			{
				bgn = ((m_nX+1) * k + i)*4;

				vcP = m_vcP + (i * m_fW) * m_vcX + (k * m_fW) * m_vcZ;
				hgt = PGX_GetHeight(ref vcP);
				uvt = PGX_GetUV(base_y, hgt);

				vcP.y = hgt;

				vtP[bgn+0] = vcP;
				vtP[bgn+1] = vcP;
				vtP[bgn+2] = vcP;
				vtP[bgn+3] = vcP;

				vtP[bgn+0].y -= m_fT;
				vtP[bgn+1]   -= m_fT * m_vcZ;
				vtP[bgn+2].y += m_fT;
				vtP[bgn+3]   += m_fT * m_vcZ;

				// makeup the texture coord x value
				vtT[bgn+0].x = uvt;
				vtT[bgn+1].x = uvt;
				vtT[bgn+2].x = uvt;
				vtT[bgn+3].x = uvt;
			}
		}

		for(i=0; i<=m_nX; ++i)
		{
			for(k=0; k<=m_nZ; ++k)
			{
				bgn = ((m_nZ+1) * i + k)*4  + (m_nX+1) * (m_nZ+1)* 4;

				vcP = m_vcP + (i * m_fW) * m_vcX + (k * m_fW) * m_vcZ;
				hgt = PGX_GetHeight(ref vcP);
				uvt = PGX_GetUV(base_y, hgt);

				vcP.y = hgt;


				vtP[bgn+0] = vcP;
				vtP[bgn+1] = vcP;
				vtP[bgn+2] = vcP;
				vtP[bgn+3] = vcP;

				vtP[bgn+0].y -= m_fT;
				vtP[bgn+1]   += m_fT * m_vcX;
				vtP[bgn+2].y += m_fT;
				vtP[bgn+3]   -= m_fT * m_vcX;

				// makeup the texture coord x value
				vtT[bgn+0].x = uvt;
				vtT[bgn+1].x = uvt;
				vtT[bgn+2].x = uvt;
				vtT[bgn+3].x = uvt;
			}
		}


		m_msh.vertices = vtP;
		m_msh.uv       = vtT;
	}
}


